@@ -8,10 +8,11 @@
 //! The types in this module are re-exported from `arti-client`: any changes
 //! here must be reflected in the version of `arti-client`.
 
-use crate::retry::RetryConfig;
+use crate::retry::DownloadSchedule;
 use crate::storage::sqlite::SqliteStore;
 use crate::Authority;
 use crate::Result;
+use tor_config::ConfigBuildError;
 use tor_netdir::fallback::FallbackDir;
 use tor_netdoc::doc::netstatus;
 
@@ -25,9 +26,9 @@ use serde::Deserialize;
 ///
 /// This type is immutable once constructed. To make one, use
 /// [`NetworkConfigBuilder`], or deserialize it from a string.
-#[derive(Deserialize, Debug, Clone, Builder)]
+#[derive(Deserialize, Debug, Clone, Builder, Eq, PartialEq)]
 #[serde(deny_unknown_fields)]
-#[builder(build_fn(validate = "Self::validate"))]
+#[builder(build_fn(validate = "Self::validate", error = "ConfigBuildError"))]
 pub struct NetworkConfig {
     /// List of locations to look in when downloading directory information,
     /// if we don't actually have a directory yet.
@@ -42,7 +43,7 @@ pub struct NetworkConfig {
     /// consensus documents.
     ///
     /// (If none are specified, we use a default list of authorities
-    /// shippedwith Arti.)
+    /// shipped with Arti.)
     #[serde(default = "crate::authority::default_authorities")]
     #[builder(default = "crate::authority::default_authorities()")]
     authorities: Vec<Authority>,
@@ -57,6 +58,16 @@ impl Default for NetworkConfig {
     }
 }
 
+impl From<NetworkConfig> for NetworkConfigBuilder {
+    fn from(cfg: NetworkConfig) -> NetworkConfigBuilder {
+        let mut builder = NetworkConfigBuilder::default();
+        builder
+            .fallback_caches(cfg.fallback_caches)
+            .authorities(cfg.authorities);
+        builder
+    }
+}
+
 impl NetworkConfig {
     /// Return a new builder to construct a NetworkConfig.
     pub fn builder() -> NetworkConfigBuilder {
@@ -74,12 +85,13 @@ impl NetworkConfig {
 
 impl NetworkConfigBuilder {
     /// Check that this builder will give a reasonable network.
-    fn validate(&self) -> std::result::Result<(), String> {
+    fn validate(&self) -> std::result::Result<(), ConfigBuildError> {
         if self.authorities.is_some() && self.fallback_caches.is_none() {
-            return Err(
-                "Non-default authorities are use, but the fallback list is not overridden"
-                    .to_string(),
-            );
+            return Err(ConfigBuildError::Inconsistent {
+                fields: vec!["authorities".to_owned(), "fallbacks".to_owned()],
+                problem: "Non-default authorities are use, but the fallback list is not overridden"
+                    .to_owned(),
+            });
         }
 
         Ok(())
@@ -91,53 +103,46 @@ impl NetworkConfigBuilder {
 ///
 /// This type is immutable once constructed. To make one, use
 /// [`DownloadScheduleConfigBuilder`], or deserialize it from a string.
-#[derive(Deserialize, Debug, Clone, Builder)]
+#[derive(Deserialize, Debug, Clone, Builder, Eq, PartialEq)]
 #[serde(deny_unknown_fields)]
+#[builder(build_fn(error = "ConfigBuildError"))]
 pub struct DownloadScheduleConfig {
     /// Top-level configuration for how to retry our initial bootstrap attempt.
     #[serde(default = "default_retry_bootstrap")]
     #[builder(default = "default_retry_bootstrap()")]
-    retry_bootstrap: RetryConfig,
+    retry_bootstrap: DownloadSchedule,
 
     /// Configuration for how to retry a consensus download.
     #[serde(default)]
     #[builder(default)]
-    retry_consensus: RetryConfig,
+    retry_consensus: DownloadSchedule,
 
     /// Configuration for how to retry an authority cert download.
     #[serde(default)]
     #[builder(default)]
-    retry_certs: RetryConfig,
+    retry_certs: DownloadSchedule,
 
     /// Configuration for how to retry a microdescriptor download.
-    #[serde(default)]
-    #[builder(default)]
-    retry_microdescs: RetryConfig,
-
-    /// Number of microdescriptor downloads to attempt in parallel
-    #[serde(default = "default_microdesc_parallelism")]
-    #[builder(default = "default_microdesc_parallelism()")]
-    microdesc_parallelism: u8,
+    #[serde(default = "default_microdesc_schedule")]
+    #[builder(default = "default_microdesc_schedule()")]
+    retry_microdescs: DownloadSchedule,
 }
 
 /// Default value for retry_bootstrap in DownloadScheduleConfig.
-fn default_retry_bootstrap() -> RetryConfig {
-    RetryConfig::new(128, std::time::Duration::new(1, 0))
+fn default_retry_bootstrap() -> DownloadSchedule {
+    DownloadSchedule::new(128, std::time::Duration::new(1, 0), 1)
 }
-/// Default value for microdesc_parallelism in DownloadScheduleConfig.
-fn default_microdesc_parallelism() -> u8 {
-    4
+
+/// Default value for microdesc_bootstrap in DownloadScheduleConfig.
+fn default_microdesc_schedule() -> DownloadSchedule {
+    DownloadSchedule::new(3, std::time::Duration::new(1, 0), 4)
 }
 
 impl Default for DownloadScheduleConfig {
     fn default() -> Self {
-        DownloadScheduleConfig {
-            retry_bootstrap: default_retry_bootstrap(),
-            retry_consensus: Default::default(),
-            retry_certs: Default::default(),
-            retry_microdescs: Default::default(),
-            microdesc_parallelism: default_microdesc_parallelism(),
-        }
+        Self::builder()
+            .build()
+            .expect("default builder setting didn't work")
     }
 }
 
@@ -148,6 +153,18 @@ impl DownloadScheduleConfig {
     }
 }
 
+impl From<DownloadScheduleConfig> for DownloadScheduleConfigBuilder {
+    fn from(cfg: DownloadScheduleConfig) -> DownloadScheduleConfigBuilder {
+        let mut builder = DownloadScheduleConfigBuilder::default();
+        builder
+            .retry_bootstrap(cfg.retry_bootstrap)
+            .retry_consensus(cfg.retry_consensus)
+            .retry_certs(cfg.retry_certs)
+            .retry_microdescs(cfg.retry_microdescs);
+        builder
+    }
+}
+
 /// Configuration type for network directory operations.
 ///
 /// This type is immutable once constructed.
@@ -155,7 +172,8 @@ impl DownloadScheduleConfig {
 /// To create an object of this type, use [`DirMgrConfigBuilder`], or
 /// deserialize it from a string. (Arti generally uses Toml for
 /// configuration, but you can use other formats if you prefer.)
-#[derive(Debug, Clone, Builder)]
+#[derive(Debug, Clone, Builder, Eq, PartialEq)]
+#[builder(build_fn(error = "ConfigBuildError"))]
 pub struct DirMgrConfig {
     /// Location to use for storing and reading current-format
     /// directory information.
@@ -237,29 +255,24 @@ impl DirMgrConfig {
 impl DownloadScheduleConfig {
     /// Return configuration for retrying our entire bootstrap
     /// operation at startup.
-    pub(crate) fn retry_bootstrap(&self) -> &RetryConfig {
+    pub(crate) fn retry_bootstrap(&self) -> &DownloadSchedule {
         &self.retry_bootstrap
     }
 
     /// Return configuration for retrying a consensus download.
-    pub(crate) fn retry_consensus(&self) -> &RetryConfig {
+    pub(crate) fn retry_consensus(&self) -> &DownloadSchedule {
         &self.retry_consensus
     }
 
     /// Return configuration for retrying an authority certificate download
-    pub(crate) fn retry_certs(&self) -> &RetryConfig {
+    pub(crate) fn retry_certs(&self) -> &DownloadSchedule {
         &self.retry_certs
     }
 
     /// Return configuration for retrying an authority certificate download
-    pub(crate) fn retry_microdescs(&self) -> &RetryConfig {
+    pub(crate) fn retry_microdescs(&self) -> &DownloadSchedule {
         &self.retry_microdescs
     }
-
-    /// Number of microdescriptor fetches to attempt in parallel
-    pub(crate) fn microdesc_parallelism(&self) -> usize {
-        self.microdesc_parallelism.max(1).into()
-    }
 }
 
 /// Helpers for initializing the fallback list.
@@ -365,18 +378,17 @@ mod test {
         let mut bld = DownloadScheduleConfig::builder();
 
         let cfg = bld.build().unwrap();
-        assert_eq!(cfg.microdesc_parallelism(), 4);
+        assert_eq!(cfg.retry_microdescs().parallelism(), 4);
         assert_eq!(cfg.retry_microdescs().n_attempts(), 3);
         assert_eq!(cfg.retry_bootstrap().n_attempts(), 128);
 
-        bld.microdesc_parallelism(0)
-            .retry_consensus(RetryConfig::new(7, Duration::new(86400, 0)))
-            .retry_bootstrap(RetryConfig::new(4, Duration::new(3600, 0)))
-            .retry_certs(RetryConfig::new(5, Duration::new(3600, 0)))
-            .retry_microdescs(RetryConfig::new(6, Duration::new(3600, 0)));
+        bld.retry_consensus(DownloadSchedule::new(7, Duration::new(86400, 0), 1))
+            .retry_bootstrap(DownloadSchedule::new(4, Duration::new(3600, 0), 1))
+            .retry_certs(DownloadSchedule::new(5, Duration::new(3600, 0), 1))
+            .retry_microdescs(DownloadSchedule::new(6, Duration::new(3600, 0), 0));
 
         let cfg = bld.build().unwrap();
-        assert_eq!(cfg.microdesc_parallelism(), 1); // gets clamped to 1
+        assert_eq!(cfg.retry_microdescs().parallelism(), 1); // gets clamped
         assert_eq!(cfg.retry_microdescs().n_attempts(), 6);
         assert_eq!(cfg.retry_bootstrap().n_attempts(), 4);
         assert_eq!(cfg.retry_consensus().n_attempts(), 7);
