@@ -20,6 +20,9 @@
 //! reading large directory objects from disk.
 //!
 //! `static` -- Try to link with a static copy of sqlite3.
+//!
+//! `routerdesc` -- (Incomplete) support for downloading and storing
+//!      router descriptors.
 
 #![deny(missing_docs)]
 #![warn(noop_method_call)]
@@ -45,6 +48,7 @@
 #![warn(clippy::option_option)]
 #![warn(clippy::rc_buffer)]
 #![deny(clippy::ref_option_ref)]
+#![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
@@ -65,7 +69,7 @@ mod storage;
 use crate::docid::{CacheUsage, ClientRequest, DocQuery};
 use crate::shared_ref::SharedMutArc;
 use crate::storage::sqlite::SqliteStore;
-use retry::RetryConfig;
+pub use retry::DownloadSchedule;
 use tor_circmgr::CircMgr;
 use tor_netdir::NetDir;
 use tor_netdoc::doc::netstatus::ConsensusFlavor;
@@ -150,7 +154,7 @@ impl<R: Runtime> DirMgr<R> {
     /// program; it's only suitable for command-line or batch tools.
     // TODO: I wish this function didn't have to be async or take a runtime.
     pub async fn load_once(runtime: R, config: DirMgrConfig) -> Result<Arc<NetDir>> {
-        let dirmgr = Arc::new(Self::from_config(config, runtime, None)?);
+        let dirmgr = Arc::new(Self::from_config(config, runtime, None, true)?);
 
         // TODO: add some way to return a directory that isn't up-to-date
         let _success = dirmgr.load_directory().await?;
@@ -188,7 +192,12 @@ impl<R: Runtime> DirMgr<R> {
         runtime: R,
         circmgr: Arc<CircMgr<R>>,
     ) -> Result<Arc<Self>> {
-        let dirmgr = Arc::new(DirMgr::from_config(config, runtime.clone(), Some(circmgr))?);
+        let dirmgr = Arc::new(DirMgr::from_config(
+            config,
+            runtime.clone(),
+            Some(circmgr),
+            false,
+        )?);
 
         // Try to load from the cache.
         let have_directory = dirmgr.load_directory().await?;
@@ -419,8 +428,8 @@ impl<R: Runtime> DirMgr<R> {
         config: DirMgrConfig,
         runtime: R,
         circmgr: Option<Arc<CircMgr<R>>>,
+        readonly: bool,
     ) -> Result<Self> {
-        let readonly = circmgr.is_none();
         let store = Mutex::new(config.open_sqlite_store(readonly)?);
         let netdir = SharedMutArc::new();
         let netdir_consensus_changed = AtomicBool::new(false);
@@ -569,6 +578,7 @@ impl<R: Runtime> DirMgr<R> {
                         .map(|(id, md)| (DocId::Microdesc(id), DocumentText::from_string(md))),
                 );
             }
+            #[cfg(feature = "routerdesc")]
             RouterDesc(digests) => result.extend(
                 store
                     .routerdescs(digests)?
@@ -597,6 +607,7 @@ impl<R: Runtime> DirMgr<R> {
                 DocQuery::Microdesc(ids) => {
                     res.push(ClientRequest::Microdescs(ids.into_iter().collect()));
                 }
+                #[cfg(feature = "routerdesc")]
                 DocQuery::RouterDesc(ids) => {
                     res.push(ClientRequest::RouterDescs(ids.into_iter().collect()));
                 }
@@ -705,7 +716,7 @@ trait DirState: Send {
     /// was any change in this state.
     ///
     /// If `storage` is provided, then we should write any state changes into
-    /// it.
+    /// it.  (We don't read from it in this method.)
     fn add_from_cache(
         &mut self,
         docs: HashMap<DocId, DocumentText>,
@@ -731,10 +742,8 @@ trait DirState: Send {
         request: &ClientRequest,
         storage: Option<&Mutex<SqliteStore>>,
     ) -> Result<bool>;
-    /// Return the number of attempts that should be made in parallel
-    /// to attempt downloads for missing documents, and a
-    /// configuration for retrying downloads.
-    fn dl_config(&self) -> Result<(usize, RetryConfig)>;
+    /// Return a configuration for attempting downloads.
+    fn dl_config(&self) -> Result<DownloadSchedule>;
     /// If possible, advance to the next state.
     fn advance(self: Box<Self>) -> Result<Box<dyn DirState>>;
     /// Return a time (if any) when downloaders should stop attempting to
@@ -749,3 +758,317 @@ trait DirState: Send {
 fn upgrade_weak_ref<T>(weak: &Weak<T>) -> Result<Arc<T>> {
     Weak::upgrade(weak).ok_or(Error::ManagerDropped)
 }
+
+#[cfg(test)]
+mod test {
+    #![allow(clippy::unwrap_used)]
+    use super::*;
+    use crate::docmeta::{AuthCertMeta, ConsensusMeta};
+    use std::time::Duration;
+    use tempfile::TempDir;
+    use tor_netdoc::doc::{authcert::AuthCertKeyIds, netstatus::Lifetime};
+
+    pub(crate) fn new_mgr<R: Runtime>(runtime: R) -> (TempDir, DirMgr<R>) {
+        let dir = TempDir::new().unwrap();
+        let config = DirMgrConfig::builder()
+            .cache_path(dir.path())
+            .build()
+            .unwrap();
+        let dirmgr = DirMgr::from_config(config, runtime, None, false).unwrap();
+
+        (dir, dirmgr)
+    }
+
+    #[test]
+    fn failing_accessors() {
+        tor_rtcompat::test_with_one_runtime!(|rt| async {
+            let (_tempdir, mgr) = new_mgr(rt);
+
+            assert!(mgr.circmgr().is_err());
+            assert!(mgr.opt_netdir().is_none());
+        });
+    }
+
+    #[test]
+    fn load_and_store_internals() {
+        tor_rtcompat::test_with_one_runtime!(|rt| async {
+            let (_tempdir, mgr) = new_mgr(rt);
+
+            let now = SystemTime::now();
+            let tomorrow = now + Duration::from_secs(86400);
+            let later = tomorrow + Duration::from_secs(86400);
+
+            // Seed the storage with a bunch of junk.
+            let d1 = [5_u8; 32];
+            let d2 = [7; 32];
+            let d3 = [42; 32];
+            let d4 = [99; 20];
+            let d5 = [12; 20];
+            let certid1 = AuthCertKeyIds {
+                id_fingerprint: d4.into(),
+                sk_fingerprint: d5.into(),
+            };
+            let certid2 = AuthCertKeyIds {
+                id_fingerprint: d5.into(),
+                sk_fingerprint: d4.into(),
+            };
+
+            {
+                let mut store = mgr.store.lock().unwrap();
+
+                store
+                    .store_microdescs(
+                        vec![
+                            ("Fake micro 1", &d1),
+                            ("Fake micro 2", &d2),
+                            ("Fake micro 3", &d3),
+                        ],
+                        now,
+                    )
+                    .unwrap();
+
+                #[cfg(feature = "routerdesc")]
+                store
+                    .store_routerdescs(vec![("Fake rd1", now, &d4), ("Fake rd2", now, &d5)])
+                    .unwrap();
+
+                store
+                    .store_authcerts(&[
+                        (
+                            AuthCertMeta::new(certid1, now, tomorrow),
+                            "Fake certificate one",
+                        ),
+                        (
+                            AuthCertMeta::new(certid2, now, tomorrow),
+                            "Fake certificate two",
+                        ),
+                    ])
+                    .unwrap();
+
+                let cmeta = ConsensusMeta::new(
+                    Lifetime::new(now, tomorrow, later).unwrap(),
+                    [102; 32],
+                    [103; 32],
+                );
+                store
+                    .store_consensus(&cmeta, ConsensusFlavor::Microdesc, false, "Fake consensus!")
+                    .unwrap();
+            }
+
+            // Try to get it with text().
+            let t1 = mgr.text(&DocId::Microdesc(d1)).unwrap().unwrap();
+            assert_eq!(t1.as_str(), Ok("Fake micro 1"));
+
+            let t2 = mgr
+                .text(&DocId::LatestConsensus {
+                    flavor: ConsensusFlavor::Microdesc,
+                    cache_usage: CacheUsage::CacheOkay,
+                })
+                .unwrap()
+                .unwrap();
+            assert_eq!(t2.as_str(), Ok("Fake consensus!"));
+
+            let t3 = mgr.text(&DocId::Microdesc([255; 32])).unwrap();
+            assert!(t3.is_none());
+
+            // Now try texts()
+            let d_bogus = DocId::Microdesc([255; 32]);
+            let res = mgr
+                .texts(vec![
+                    DocId::Microdesc(d2),
+                    DocId::Microdesc(d3),
+                    d_bogus,
+                    DocId::AuthCert(certid2),
+                    #[cfg(feature = "routerdesc")]
+                    DocId::RouterDesc(d5),
+                ])
+                .unwrap();
+            assert_eq!(
+                res.get(&DocId::Microdesc(d2)).unwrap().as_str(),
+                Ok("Fake micro 2")
+            );
+            assert_eq!(
+                res.get(&DocId::Microdesc(d3)).unwrap().as_str(),
+                Ok("Fake micro 3")
+            );
+            assert!(res.get(&d_bogus).is_none());
+            assert_eq!(
+                res.get(&DocId::AuthCert(certid2)).unwrap().as_str(),
+                Ok("Fake certificate two")
+            );
+            #[cfg(feature = "routerdesc")]
+            assert_eq!(
+                res.get(&DocId::RouterDesc(d5)).unwrap().as_str(),
+                Ok("Fake rd2")
+            );
+        });
+    }
+
+    #[test]
+    fn make_consensus_request() {
+        tor_rtcompat::test_with_one_runtime!(|rt| async {
+            let (_tempdir, mgr) = new_mgr(rt);
+
+            let now = SystemTime::now();
+            let tomorrow = now + Duration::from_secs(86400);
+            let later = tomorrow + Duration::from_secs(86400);
+
+            // Try with an empty store.
+            let req = mgr
+                .make_consensus_request(ConsensusFlavor::Microdesc)
+                .unwrap();
+            match req {
+                ClientRequest::Consensus(r) => {
+                    assert_eq!(r.old_consensus_digests().count(), 0);
+                    assert_eq!(r.last_consensus_date(), None);
+                }
+                _ => panic!("Wrong request type"),
+            }
+
+            // Add a fake consensus record.
+            let d_prev = [42; 32];
+            {
+                let mut store = mgr.store.lock().unwrap();
+
+                let cmeta = ConsensusMeta::new(
+                    Lifetime::new(now, tomorrow, later).unwrap(),
+                    d_prev,
+                    [103; 32],
+                );
+                store
+                    .store_consensus(&cmeta, ConsensusFlavor::Microdesc, false, "Fake consensus!")
+                    .unwrap();
+            }
+
+            // Now try again.
+            let req = mgr
+                .make_consensus_request(ConsensusFlavor::Microdesc)
+                .unwrap();
+            match req {
+                ClientRequest::Consensus(r) => {
+                    let ds: Vec<_> = r.old_consensus_digests().collect();
+                    assert_eq!(ds.len(), 1);
+                    assert_eq!(ds[0], &d_prev);
+                    assert_eq!(r.last_consensus_date(), Some(now));
+                }
+                _ => panic!("Wrong request type"),
+            }
+        });
+    }
+
+    #[test]
+    fn make_other_requests() {
+        tor_rtcompat::test_with_one_runtime!(|rt| async {
+            use rand::Rng;
+            let (_tempdir, mgr) = new_mgr(rt);
+
+            let certid1 = AuthCertKeyIds {
+                id_fingerprint: [99; 20].into(),
+                sk_fingerprint: [100; 20].into(),
+            };
+            let mut rng = rand::thread_rng();
+            #[cfg(feature = "routerdesc")]
+            let rd_ids: Vec<[u8; 20]> = (0..1000).map(|_| rng.gen()).collect();
+            let md_ids: Vec<[u8; 32]> = (0..1000).map(|_| rng.gen()).collect();
+
+            // Try an authcert.
+            let query = DocQuery::AuthCert(vec![certid1]);
+            let reqs = mgr.query_into_requests(query).unwrap();
+            assert_eq!(reqs.len(), 1);
+            let req = &reqs[0];
+            if let ClientRequest::AuthCert(r) = req {
+                assert_eq!(r.keys().next(), Some(&certid1));
+            } else {
+                panic!();
+            }
+
+            // Try a bunch of mds.
+            let query = DocQuery::Microdesc(md_ids);
+            let reqs = mgr.query_into_requests(query).unwrap();
+            assert_eq!(reqs.len(), 2);
+            assert!(matches!(reqs[0], ClientRequest::Microdescs(_)));
+
+            // Try a bunch of rds.
+            #[cfg(feature = "routerdesc")]
+            {
+                let query = DocQuery::RouterDesc(rd_ids);
+                let reqs = mgr.query_into_requests(query).unwrap();
+                assert_eq!(reqs.len(), 2);
+                assert!(matches!(reqs[0], ClientRequest::RouterDescs(_)));
+            }
+        });
+    }
+
+    #[test]
+    fn expand_response() {
+        tor_rtcompat::test_with_one_runtime!(|rt| async {
+            let (_tempdir, mgr) = new_mgr(rt);
+
+            // Try a simple request: nothing should happen.
+            let q = DocId::Microdesc([99; 32]).into();
+            let r = &mgr.query_into_requests(q).unwrap()[0];
+            let expanded = mgr.expand_response_text(r, "ABC".to_string());
+            assert_eq!(&expanded.unwrap(), "ABC");
+
+            // Try a consensus response that doesn't look like a diff in
+            // response to a query that doesn't ask for one.
+            let latest_id = DocId::LatestConsensus {
+                flavor: ConsensusFlavor::Microdesc,
+                cache_usage: CacheUsage::CacheOkay,
+            };
+            let q: DocQuery = latest_id.into();
+            let r = &mgr.query_into_requests(q.clone()).unwrap()[0];
+            let expanded = mgr.expand_response_text(r, "DEF".to_string());
+            assert_eq!(&expanded.unwrap(), "DEF");
+
+            // Now stick some metadata and a string into the storage so that
+            // we can ask for a diff.
+            {
+                let mut store = mgr.store.lock().unwrap();
+                let now = SystemTime::now();
+                let day = Duration::from_secs(86400);
+                let d_in = [0x99; 32]; // This one, we can fake.
+                let cmeta = ConsensusMeta::new(
+                    Lifetime::new(now, now + day, now + 2 * day).unwrap(),
+                    d_in,
+                    d_in,
+                );
+                store
+                    .store_consensus(
+                        &cmeta,
+                        ConsensusFlavor::Microdesc,
+                        false,
+                        "line 1\nline2\nline 3\n",
+                    )
+                    .unwrap();
+            }
+
+            // Try expanding something that isn't a consensus, even if we'd like
+            // one.
+            let r = &mgr.query_into_requests(q).unwrap()[0];
+            let expanded = mgr.expand_response_text(r, "hello".to_string());
+            assert_eq!(&expanded.unwrap(), "hello");
+
+            // Finally, try "expanding" a diff (by applying it and checking the digest.
+            let diff = "network-status-diff-version 1
+hash 9999999999999999999999999999999999999999999999999999999999999999 8382374ca766873eb0d2530643191c6eaa2c5e04afa554cbac349b5d0592d300
+2c
+replacement line
+.
+".to_string();
+            let expanded = mgr.expand_response_text(r, diff);
+
+            assert_eq!(expanded.unwrap(), "line 1\nreplacement line\nline 3\n");
+
+            // If the digest is wrong, that should get rejected.
+            let diff = "network-status-diff-version 1
+hash 9999999999999999999999999999999999999999999999999999999999999999 9999999999999999999999999999999999999999999999999999999999999999
+2c
+replacement line
+.
+".to_string();
+            let expanded = mgr.expand_response_text(r, diff);
+            assert!(expanded.is_err());
+        });
+    }
+}
