@@ -22,7 +22,7 @@ use tracing::{info, warn};
 
 use crate::{
     docmeta::{AuthCertMeta, ConsensusMeta},
-    retry::RetryConfig,
+    retry::DownloadSchedule,
     shared_ref::SharedMutArc,
     storage::sqlite::SqliteStore,
     CacheUsage, ClientRequest, DirMgrConfig, DirState, DocId, DocumentText, Error, Readiness,
@@ -169,9 +169,9 @@ impl<DM: WriteNetDir> DirState for GetConsensusState<DM> {
     fn can_advance(&self) -> bool {
         self.next.is_some()
     }
-    fn dl_config(&self) -> Result<(usize, RetryConfig)> {
+    fn dl_config(&self) -> Result<DownloadSchedule> {
         if let Some(wd) = Weak::upgrade(&self.writedir) {
-            Ok((1, *wd.config().schedule().retry_consensus()))
+            Ok(*wd.config().schedule().retry_consensus())
         } else {
             Err(Error::ManagerDropped)
         }
@@ -229,7 +229,7 @@ impl<DM: WriteNetDir> DirState for GetConsensusState<DM> {
 impl<DM: WriteNetDir> GetConsensusState<DM> {
     /// Helper: try to set the current consensus text from an input
     /// string `text`.  Refuse it if the authorities could never be
-    /// correct, or if it is illformed.
+    /// correct, or if it is ill-formed.
     fn add_consensus_text(
         &mut self,
         from_cache: bool,
@@ -248,7 +248,7 @@ impl<DM: WriteNetDir> GetConsensusState<DM> {
         };
 
         // Check out what authorities we believe in, and see if enough
-        // of them are purported to have singed this consensus.
+        // of them are purported to have signed this consensus.
         let n_authorities = self.authority_ids.len() as u16;
         let unvalidated = unvalidated.set_n_authorities(n_authorities);
 
@@ -335,9 +335,9 @@ impl<DM: WriteNetDir> DirState for GetCertsState<DM> {
     fn can_advance(&self) -> bool {
         self.unvalidated.key_is_correct(&self.certs[..]).is_ok()
     }
-    fn dl_config(&self) -> Result<(usize, RetryConfig)> {
+    fn dl_config(&self) -> Result<DownloadSchedule> {
         if let Some(wd) = Weak::upgrade(&self.writedir) {
-            Ok((1, *wd.config().schedule().retry_certs()))
+            Ok(*wd.config().schedule().retry_certs())
         } else {
             Err(Error::ManagerDropped)
         }
@@ -350,7 +350,7 @@ impl<DM: WriteNetDir> DirState for GetCertsState<DM> {
         let mut changed = false;
         // Here we iterate over the documents we want, taking them from
         // our input and remembering them.
-        for id in self.missing_docs().iter() {
+        for id in &self.missing_docs() {
             if let Some(cert) = docs.get(id) {
                 let parsed = AuthCert::parse(cert.as_str()?)?.check_signature()?;
                 let now = current_time(&self.writedir)?;
@@ -554,6 +554,22 @@ impl<DM: WriteNetDir> GetMicrodescsState<DM> {
         }
         false
     }
+
+    /// Mark the consensus that we're getting MDs for as non-pending in the
+    /// storage.
+    ///
+    /// Called when a consensus is no longer pending.
+    fn mark_consensus_usable(&self, storage: Option<&Mutex<SqliteStore>>) -> Result<()> {
+        if let Some(store) = storage {
+            let mut store = store.lock().expect("Directory storage lock poisoned");
+            info!("Marked consensus usable.");
+            store.mark_consensus_usable(&self.meta)?;
+            // Now that a consensus is usable, older consensuses may
+            // need to expire.
+            store.expire_all()?;
+        }
+        Ok(())
+    }
 }
 
 impl<DM: WriteNetDir> DirState for GetMicrodescsState<DM> {
@@ -575,12 +591,9 @@ impl<DM: WriteNetDir> DirState for GetMicrodescsState<DM> {
     fn can_advance(&self) -> bool {
         false
     }
-    fn dl_config(&self) -> Result<(usize, RetryConfig)> {
+    fn dl_config(&self) -> Result<DownloadSchedule> {
         if let Some(wd) = Weak::upgrade(&self.writedir) {
-            Ok((
-                wd.config().schedule().microdesc_parallelism(),
-                *wd.config().schedule().retry_microdescs(),
-            ))
+            Ok(*wd.config().schedule().retry_microdescs())
         } else {
             Err(Error::ManagerDropped)
         }
@@ -610,17 +623,13 @@ impl<DM: WriteNetDir> DirState for GetMicrodescsState<DM> {
 
         let changed = !microdescs.is_empty();
         if self.register_microdescs(microdescs) {
-            if let Some(store) = storage {
-                let mut store = store.lock().expect("Directory storage lock poisoned");
-                info!("Marked consensus usable.");
-                store.mark_consensus_usable(&self.meta)?;
-                // DOCDOC: explain why we're doing this here.
-                store.expire_all()?;
-            }
+            // Just stopped being pending.
+            self.mark_consensus_usable(storage)?;
         }
 
         Ok(changed)
     }
+
     fn add_from_download(
         &mut self,
         text: &str,
@@ -634,7 +643,6 @@ impl<DM: WriteNetDir> DirState for GetMicrodescsState<DM> {
         };
         let mut new_mds = Vec::new();
         for anno in MicrodescReader::new(text, &AllowAnnotations::AnnotationsNotAllowed).flatten() {
-            dbg!("Adding 1 from download");
             let txt = anno
                 .within(text)
                 .expect("annotation not from within text as expected");
@@ -665,14 +673,8 @@ impl<DM: WriteNetDir> DirState for GetMicrodescsState<DM> {
             }
         }
         if self.register_microdescs(new_mds.into_iter().map(|(_, md)| md)) {
-            // oh hey, this is no longer pending.
-            if let Some(store) = storage {
-                let mut store = store.lock().expect("Directory storage lock poisoned");
-                info!("Marked consensus usable.");
-                store.mark_consensus_usable(&self.meta)?;
-                // DOCDOC: explain why we're doing this here.
-                store.expire_all()?;
-            }
+            // Just stopped being pending.
+            self.mark_consensus_usable(storage)?;
         }
         Ok(true)
     }
@@ -685,7 +687,7 @@ impl<DM: WriteNetDir> DirState for GetMicrodescsState<DM> {
     fn reset(self: Box<Self>) -> Result<Box<dyn DirState>> {
         Ok(Box::new(GetConsensusState::new(
             self.writedir,
-            CacheUsage::MustDownload,
+            CacheUsage::MustDownload, // XXXX I believe this is wrong?
         )?))
     }
 }
@@ -740,13 +742,15 @@ fn current_time<DM: WriteNetDir>(writedir: &Weak<DM>) -> Result<SystemTime> {
 #[cfg(test)]
 mod test {
     #![allow(clippy::unwrap_used)]
+    #![allow(clippy::cognitive_complexity)]
     use super::*;
-    use crate::Authority;
+    use crate::{Authority, DownloadScheduleConfig};
     use std::convert::TryInto;
     use std::sync::{
         atomic::{self, AtomicBool},
         Arc,
     };
+    use tempfile::TempDir;
     use time::macros::datetime;
     use tor_netdoc::doc::authcert::AuthCertKeyIds;
 
@@ -773,6 +777,14 @@ mod test {
         }
     }
 
+    /// Makes a memory-backed SqliteStore.
+    fn temp_store() -> (TempDir, Mutex<SqliteStore>) {
+        let tempdir = TempDir::new().unwrap();
+        let conn = rusqlite::Connection::open_in_memory().unwrap();
+        let store = SqliteStore::from_conn(conn, tempdir.path()).unwrap();
+        (tempdir, Mutex::new(store))
+    }
+
     struct DirRcv {
         cfg: DirMgrConfig,
         netdir: SharedMutArc<NetDir>,
@@ -889,6 +901,8 @@ mod test {
     fn get_consensus_state() {
         let rcv = Arc::new(DirRcv::new(test_time(), None));
 
+        let (_tempdir, store) = temp_store();
+
         let mut state =
             GetConsensusState::new(Arc::downgrade(&rcv), CacheUsage::CacheOkay).unwrap();
 
@@ -903,6 +917,11 @@ mod test {
         // Basic properties: it doesn't want to reset.
         assert!(state.reset_time().is_none());
 
+        // Download configuration is simple: only 1 request can be done in
+        // parallel.  It uses a consensus retry schedule.
+        let retry = state.dl_config().unwrap();
+        assert_eq!(&retry, DownloadScheduleConfig::default().retry_consensus());
+
         // Do we know what we want?
         let docs = state.missing_docs();
         assert_eq!(docs.len(), 1);
@@ -919,12 +938,25 @@ mod test {
         // Now suppose that we get some complete junk from a download.
         let req = tor_dirclient::request::ConsensusRequest::new(ConsensusFlavor::Microdesc);
         let req = crate::docid::ClientRequest::Consensus(req);
-        let outcome = state.add_from_download("this isn't a consensus", &req, None);
+        let outcome = state.add_from_download("this isn't a consensus", &req, Some(&store));
         assert!(matches!(outcome, Err(Error::NetDocError(_))));
+        // make sure it wasn't stored...
+        assert!(store
+            .lock()
+            .unwrap()
+            .latest_consensus(ConsensusFlavor::Microdesc, None)
+            .unwrap()
+            .is_none());
 
         // Now try again, with a real consensus... but the wrong authorities.
-        let outcome = state.add_from_download(CONSENSUS, &req, None);
+        let outcome = state.add_from_download(CONSENSUS, &req, Some(&store));
         assert!(matches!(outcome, Err(Error::UnrecognizedAuthorities)));
+        assert!(store
+            .lock()
+            .unwrap()
+            .latest_consensus(ConsensusFlavor::Microdesc, None)
+            .unwrap()
+            .is_none());
 
         // Great. Change the receiver to use a configuration where these test
         // authorities are recognized.
@@ -932,11 +964,19 @@ mod test {
 
         let mut state =
             GetConsensusState::new(Arc::downgrade(&rcv), CacheUsage::CacheOkay).unwrap();
-        let outcome = state.add_from_download(CONSENSUS, &req, None);
+        let outcome = state.add_from_download(CONSENSUS, &req, Some(&store));
         assert!(outcome.unwrap());
+        assert!(store
+            .lock()
+            .unwrap()
+            .latest_consensus(ConsensusFlavor::Microdesc, None)
+            .unwrap()
+            .is_some());
 
         // And with that, we should be asking for certificates
         assert!(state.can_advance());
+        assert_eq!(&state.describe(), "About to fetch certificates.");
+        assert_eq!(state.missing_docs(), Vec::new());
         let next = Box::new(state).advance().unwrap();
         assert_eq!(
             &next.describe(),
@@ -968,6 +1008,7 @@ mod test {
             (rcv, Box::new(state).advance().unwrap())
         }
 
+        let (_tempdir, store) = temp_store();
         let (_rcv, mut state) = new_getcerts_state();
         // Basic properties: description, status, reset time.
         assert_eq!(
@@ -979,6 +1020,8 @@ mod test {
         assert!(!state.is_ready(Readiness::Usable));
         let consensus_expires = datetime!(2020-08-07 12:43:20 UTC).into();
         assert_eq!(state.reset_time(), Some(consensus_expires));
+        let retry = state.dl_config().unwrap();
+        assert_eq!(&retry, DownloadScheduleConfig::default().retry_certs());
 
         // Check that we get the right list of missing docs.
         let missing = state.missing_docs();
@@ -1006,20 +1049,32 @@ mod test {
         let mut req = tor_dirclient::request::AuthCertRequest::new();
         req.push(authcert_id_5696()); // it's the wrong id.
         let req = ClientRequest::AuthCert(req);
-        let outcome = state.add_from_download(AUTHCERT_5A23, &req, None);
+        let outcome = state.add_from_download(AUTHCERT_5A23, &req, Some(&store));
         assert!(!outcome.unwrap()); // no error, but nothing changed.
         let missing2 = state.missing_docs();
         assert_eq!(missing, missing2); // No change.
+        assert!(store
+            .lock()
+            .unwrap()
+            .authcerts(&[authcert_id_5a23()])
+            .unwrap()
+            .is_empty());
 
         // Now try to add the other from a download ... for real!
         let mut req = tor_dirclient::request::AuthCertRequest::new();
         req.push(authcert_id_5a23()); // Right idea this time!
         let req = ClientRequest::AuthCert(req);
-        let outcome = state.add_from_download(AUTHCERT_5A23, &req, None);
+        let outcome = state.add_from_download(AUTHCERT_5A23, &req, Some(&store));
         assert!(outcome.unwrap()); // No error, _and_ something changed!
         let missing3 = state.missing_docs();
         assert!(missing3.is_empty());
         assert!(state.can_advance());
+        assert!(!store
+            .lock()
+            .unwrap()
+            .authcerts(&[authcert_id_5a23()])
+            .unwrap()
+            .is_empty());
 
         let next = state.advance().unwrap();
         assert_eq!(
@@ -1027,6 +1082,11 @@ mod test {
             "Downloading microdescriptors (we are missing 6)."
         );
 
+        // If we start from scratch and reset, we're back in GetConsensus.
+        let (_rcv, state) = new_getcerts_state();
+        let state = state.reset().unwrap();
+        assert_eq!(&state.describe(), "Looking for a consensus.");
+
         // TODO: I'd like even more tests to make sure that we never
         // accept a certificate for an authority we don't believe in.
     }
@@ -1049,9 +1109,13 @@ mod test {
             base64::decode(s).unwrap().try_into().unwrap()
         }
 
-        let (_rcv, mut state) = new_getmicrodescs_state();
+        // If we start from scratch and reset, we're back in GetConsensus.
+        let (_rcv, state) = new_getmicrodescs_state();
+        let state = Box::new(state).reset().unwrap();
+        assert_eq!(&state.describe(), "Downloading a consensus.");
 
         // Check the basics.
+        let (_rcv, mut state) = new_getmicrodescs_state();
         assert_eq!(
             &state.describe(),
             "Downloading microdescriptors (we are missing 4)."
@@ -1066,6 +1130,8 @@ mod test {
             assert!(reset_time >= fresh_until);
             assert!(reset_time <= valid_until);
         }
+        let retry = state.dl_config().unwrap();
+        assert_eq!(&retry, DownloadScheduleConfig::default().retry_microdescs());
 
         // Now check whether we're missing all the right microdescs.
         let missing = state.missing_docs();
@@ -1082,11 +1148,12 @@ mod test {
         }
 
         // Try adding a microdesc from the cache.
+        let (_tempdir, store) = temp_store();
         let doc1: crate::storage::InputString = md_text.get(&md1).unwrap().clone().into();
         let docs = vec![(DocId::Microdesc(md1), doc1.into())]
             .into_iter()
             .collect();
-        let outcome = state.add_from_cache(docs, None);
+        let outcome = state.add_from_cache(docs, Some(&store));
         assert!(outcome.unwrap()); // successfully loaded one MD.
         assert!(!state.can_advance());
         assert!(!state.is_ready(Readiness::Complete));
@@ -1105,11 +1172,19 @@ mod test {
             req.push(md_digest);
         }
         let req = ClientRequest::Microdescs(req);
-        print!("{}", &response);
-        let outcome = state.add_from_download(response.as_str(), &req, None);
+        let outcome = state.add_from_download(response.as_str(), &req, Some(&store));
         assert!(outcome.unwrap()); // successfully loaded MDs
         assert!(state.is_ready(Readiness::Complete));
         assert!(state.is_ready(Readiness::Usable));
+        assert_eq!(
+            store
+                .lock()
+                .unwrap()
+                .microdescs(&[md2, md3, md4])
+                .unwrap()
+                .len(),
+            3
+        );
 
         let missing = state.missing_docs();
         assert!(missing.is_empty());
